@@ -12,7 +12,15 @@ class SmtpSSLTest(unittest.TestCase):
 
     def test_connect(self):
         support.get_attribute(smtplib, 'SMTP_SSL')
-        server = smtplib.SMTP_SSL(self.testServer, self.remotePort)
+        with support.transient_internet(self.testServer):
+            server = smtplib.SMTP_SSL(self.testServer, self.remotePort)
+        server.ehlo()
+        server.quit()
+
+    def test_connect_default_port(self):
+        support.get_attribute(smtplib, 'SMTP_SSL')
+        with support.transient_internet(self.testServer):
+            server = smtplib.SMTP_SSL(self.testServer)
         server.ehlo()
         server.quit()
 
