@@ -1,6 +1,6 @@
 # -*-mode: python; fill-column: 75; tab-width: 8 -*-
 #
-# $Id: tix.py 63418 2008-05-17 18:39:55Z georg.brandl $
+# $Id$
 #
 # Tix.py -- Tix widget wrappers.
 #
@@ -148,7 +148,7 @@ class tixCommand:
         extensions) exist, then the image type is chosen according to the
         depth of the X display: xbm images are chosen on monochrome
         displays and color images are chosen on color displays. By using
-        tix_ getimage, you can advoid hard coding the pathnames of the
+        tix_ getimage, you can avoid hard coding the pathnames of the
         image files in your application. When successful, this command
         returns the name of the newly created image, which can be used to
         configure the -image option of the Tk and Tix widgets.
@@ -156,7 +156,7 @@ class tixCommand:
         return self.tk.call('tix', 'getimage', name)
 
     def tix_option_get(self, name):
-        """Gets  the options  manitained  by  the  Tix
+        """Gets  the options  maintained  by  the  Tix
         scheme mechanism. Available options include:
 
             active_bg       active_fg      bg
@@ -253,10 +253,10 @@ class Form:
         return self.tk.call('tixForm', 'info', self._w, option)
 
     def slaves(self):
-        return map(self._nametowidget,
-                   self.tk.splitlist(
+        return [self._nametowidget(x) for x in
+                self.tk.splitlist(
                        self.tk.call(
-                       'tixForm', 'slaves', self._w)))
+                       'tixForm', 'slaves', self._w))]
 
 
 
@@ -293,7 +293,7 @@ class TixWidget(tkinter.Widget):
         else:
             static_options = ['options']
 
-        for k,v in cnf.items()[:]:
+        for k,v in list(cnf.items()):
             if k in static_options:
                 extra = extra + ('-' + k, v)
                 del cnf[k]
@@ -448,7 +448,7 @@ class TixSubWidget(TixWidget):
         # we must be careful not to destroy the frame widget since this
         # also destroys the parent NoteBook thus leading to an exception
         # in Tkinter when it finally calls Tcl to destroy the NoteBook
-        for c in self.children.values(): c.destroy()
+        for c in list(self.children.values()): c.destroy()
         if self._name in self.master.children:
             del self.master.children[self._name]
         if self._name in self.master.subwidget_list:
@@ -561,7 +561,7 @@ class ButtonBox(TixWidget):
 
 class ComboBox(TixWidget):
     """ComboBox - an Entry field with a dropdown menu. The user can select a
-    choice by either typing in the entry subwdget or selecting from the
+    choice by either typing in the entry subwidget or selecting from the
     listbox subwidget.
 
     Subwidget       Class
@@ -854,7 +854,7 @@ class HList(TixWidget):
     """HList - Hierarchy display  widget can be used to display any data
     that have a hierarchical structure, for example, file system directory
     trees. The list entries are indented and connected by branch lines
-    according to their places in the hierachy.
+    according to their places in the hierarchy.
 
     Subwidgets - None"""
 
@@ -1509,7 +1509,7 @@ class TList(TixWidget):
         self.tk.call(self._w, 'yview', *args)
 
 class Tree(TixWidget):
-    """Tree - The tixTree widget can be used to display hierachical
+    """Tree - The tixTree widget can be used to display hierarchical
     data in a tree form. The user can adjust
     the view of the tree by opening or closing parts of the tree."""
 
